﻿/*===========================================
    Backgrounds for this sample are powered by ThinkGeo Cloud Maps and require
    a Client ID and Secret. These were sent to you via email when you signed up
    with ThinkGeo, or you can register now at https://cloud.thinkgeo.com.
===========================================*/

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Documents;
using ThinkGeo.Core;

namespace DisplayIsoLines
{
    public partial class Sample
    {
        private string wellDepthPointDataFilePath = @"../../Data/Result.csv";
        private Dictionary<PointShape, double> _wellDepthPointData;
        private GridCell[,] _gridCellMatrix;
        private readonly string[] _colorStrings = {
            "#000080",
            "#800000"
        };

        readonly List<double> _isoLineLevels = new List<double>
        {
            //if add this to isoLineLevels, it fix the issue, I don't know why
            //0.0, 
            10.0,
            20.0
        };
        private readonly RectangleShape _extent =
            new PolygonShape("POLYGON((450812 346963,450812 332863,461462 332863,461462 346963,450812 346963))")
                .GetBoundingBox();

        public Sample()
        {
            InitializeComponent();
        }

        private void MapLoaded(object sender, RoutedEventArgs e)
        {
            Map1.MapUnit = GeographyUnit.Meter;
            Map1.CurrentExtent = _extent;
            _wellDepthPointData = GetWellDepthPointDataFromCSV(wellDepthPointDataFilePath);
            CreateGridCellMatrix();
            GetGridIsoLineLayer();
            Map1.Refresh();
        }

        private static Dictionary<PointShape, double> GetWellDepthPointDataFromCSV(string csvFilePath)
        {
            StreamReader streamReader = null;
            var wellDataPoints = new Dictionary<PointShape, double>();

            try
            {
                streamReader = new StreamReader(csvFilePath);
                var headline = streamReader.ReadLine();
                while (!streamReader.EndOfStream)
                {
                    var line = streamReader.ReadLine();

                    var parts = line.Split(',');
                    wellDataPoints.Add(new PointShape(double.Parse(parts[0]), double.Parse(parts[1])), double.Parse(parts[2]));
                }
            }
            finally
            {
                streamReader?.Close();
            }
            return wellDataPoints;
        }

        private void CreateGridCellMatrix()
        {
            //Get the current extent since we use that to gerenate the grid.  Of course this is just for
            //the demo and in the real world you can use any extent you like
            RectangleShape currentDrawingExtent = _extent;
            //Calculate the cell size based on how many rows and columns you specified
            double cellSize = 50;
            //Greate the grid definition based on the extent, cell size etc.
            GridDefinition gridDefinition = new GridDefinition(currentDrawingExtent, cellSize, 10000000, _wellDepthPointData);

            //Generate the grid based on Inverse Distance Weighted interpolation model.  You can define your own model if needed.
            _gridCellMatrix = GridFeatureSource.GenerateGridMatrix(gridDefinition, new InverseDistanceWeightedGridInterpolationModel(5, cellSize * 4));
        }


        private void GetGridIsoLineLayer()
        {
            
            Map1.Overlays.Clear();

            var sqlIsoLineLayer = new SqlTypesIsoLineLayer(_gridCellMatrix, _isoLineLevels)
            {
                IsoLineType = IsoLineType.ClosedLinesAsPolygons,
                DataValueColumnName = "ContourValue"
            };
            var lineFeatures = sqlIsoLineLayer.GetIsoLineFeatures();

            //Todo Uncomment this to see what we want
            //var lineFeatures = IsoLineLayer.GetIsoFeatures(_gridCellMatrix, _isoLineLevels, "ContourValue",  IsoLineType.ClosedLinesAsPolygons);
           
            LayerOverlay overlay = new LayerOverlay
            {
                DrawingQuality = DrawingQuality.HighQuality
            };
            Map1.Overlays.Add("isoLineOverlay", overlay);

            var isoLineLevel = new List<double>() { 10.0, 20.0};
            var dic = lineFeatures.GroupBy(f =>
            {
                var gridCellValue = double.Parse(f.ColumnValues["ContourValue"]);
                if (isoLineLevel.Any(l => l >= gridCellValue))
                    return isoLineLevel.First(l => l >= gridCellValue);
                return -1;
            }).ToDictionary(d => d.Key, d => d);


            for (var i = 0; i < isoLineLevel.Count; i++)
            {
                var lineFeatureLayer = new InMemoryFeatureLayer();
                var polygonFeatureLayer = new InMemoryFeatureLayer();

                overlay.Layers.Add(lineFeatureLayer);
                overlay.Layers.Add(polygonFeatureLayer);

                var set = new ZoomLevelSet();
                lineFeatureLayer.ZoomLevelSet = set;
                polygonFeatureLayer.ZoomLevelSet = set;

                set.ZoomLevel01.DefaultAreaStyle = new AreaStyle(new GeoPen(GeoColors.LightBlue, 1),
                    new GeoSolidBrush(GeoColor.FromArgb(255, GeoColor.FromHtml(_colorStrings[i]))));
                set.ZoomLevel01.DefaultLineStyle =
                    new LineStyle(new GeoPen(GeoColor.FromHtml(_colorStrings[i]), 3));
                set.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                foreach (var feature in dic[isoLineLevel[i]])
                {
                    lineFeatureLayer.InternalFeatures.Add(feature);
                }
            }
        }

    }
}